%  This code creates an L-infinity minimization problem, and demonstrates 
%  how to use the code 'pdhg_Linfinity' to solve it using adaptive PDHG.
%  The minimization problem is.
%            min_u    ||u||_inf subject to ||Du-b||_2<eps
%  where D is a data matrix, u is the unknown vector, and b contains
%  measurements.
%  This File requires that 'pdhg_adaptive.m' and 'pdhg_Linfinity.m' be in 
%  your current path. 
%
%  For an explanation of PDHG and how it applies to this problem, see
%    "Adaptive Primal-Dual Hybrid Gradient Methods for Saddle-Point
%    Problems"  available at <http://arxiv.org/abs/1305.0546>
%


%%  Problem Dimensions
N = 512; % length of representation
M = 100; % length of vector being represented
% The signal to be represented - use a random vector
b = randn(M,1);
%   The accuracy of the representation
eps = 0; % The constraint is ||Dx-b||<eps


%% Seedn random number generator
randn('seed',0);
rand('seed',0);

%% Create a frame (sub-sampled FFT)
  % Create sub-sampling and injection operators
ind = randperm(N-1)+1;
samps = ind(1:M-1);
samps = [1 sort(samps)];
subindex = @(A,r) A(r);      % An anonymous function to index a matrix
expand = sparse(samps,1:M,ones(M,1), N,M); % maps sub-sampled vector in larger vector
 % Create a sub-sampled DFT 
D = @(x) subindex(fft(x)/sqrt(N),samps); % Note: divide by sqrt(N) to make the transform unitary
Dt = @(x) ifft(expand*x)*sqrt(N);

%% Call Solver
[sol, out] = pdhg_Linfinity(D,Dt,b,eps);


%% plot results
% show original signal
figure('position',[500 500 1000 300])
subplot(1,3,1);
plot(abs(b));
title('Original Signal');
% show L-infinity solution
subplot(1,3,2);
plot(abs(sol));
title('Recovered/Democratic Signal');
%  Plot the residuals, which measure convergence
subplot(1,3,3);
semilogy(out.p,'b');
hold on;
semilogy(out.d,'r');
hold off;
legend('Primal Residual','Dual Residual');
title('Convergence Curves');
