% Load and solve a linear program using adaptive PDHG.  The problem 
% solved is of the form
%            minimixe     <f,x>
%            subject to
%                     Ain*x<=bin     (inequalities)
%                     Aeq*x=beq      (equalities)
%                     lb <= x <= ub  (lower/upper bound)

%  This File requires that 'pdhg_lp' and 'pdhg_adaptive.m' be in your 
%  current path. 
%
%  For an explanation of PDHG and how it applies to this problem, see
%    "Adaptive Primal-Dual Hybrid Gradient Methods for Saddle-Point
%    Problems"  available at <http://arxiv.org/abs/1305.0546>
%


%% Seedn random number generator
%randn('seed',0);
%rand('seed',0);

%% Load the problem sc50b, which ships with matlab
%% Note: 'f' is defined by sc50b, so we don't define it here
% load sc50b;
% Ain=A;
% bin=b;
% Aeq=Aeq;
% beq=beq;
% ub = 1e100;
% lb = 0;

%%  Create random linear programming problem
N = 100;  % number of variables
M = 20;   % number of constraints
f = rand(N,1);  % The objective
x = rand(N,1);  % create random positive vector
Ain = rand(M,N); % Create constraint matrix
bin = Ain*x;    % create constraint right hand side.  Note:  This construction guarantees that the problem is feasible
Aeq = randn(1,N);       % There are no equality constraints
beq = Aeq*x;
ub = 100*ones(N,1);       %  Solution must be less than 100
lb = 0*ones(N,1);         %  Solution must be greater than 100


%% Record exact solution
fprintf('\tComputing Exact Solution to LP\n');
tic;
exact = linprog(f,Ain,bin,Aeq,beq,lb,ub);
time = toc;
fprintf('\t\tlinprog solver took %f seconds\n', time);

fprintf('\tCalling the PDHG solver\n');
opts = [];
[ sol, outs ] = pdhg_lp( f,Ain,bin,Aeq,beq, ub, lb, opts );
fprintf('Relative Error = %f\n',norm(sol-exact)/norm(exact));

%  Plot convergence curves using the residuals
figure;
semilogy(outs.p,'g')
hold on;
semilogy(outs.d,'r')
hold off;
legend('primal residual','dual residual')
title('Line programming convergence curves')
