%  Create a synthetic sparse regression problem, and solve using
%  using adaptive PDHG.  The problem solved is
%            min_u    |u| + lambda || Du - b ||
%  where |*| denotes the l1 norm, and ||*|| denotes the l2 norm.
%  This File requires that 'pdhg_adaptive.m' and 'pdhg_scaledLasso.m' be in 
%  your current path. 
%
%  For an explanation of PDHG and how it applies to this problem, see
%    "Adaptive Primal-Dual Hybrid Gradient Methods for Saddle-Point
%    Problems"  available at <http://arxiv.org/abs/1305.0546>
%

%%  Seed the random number generator
randn('seed',0); rand('seed',0);  

%%  Parameters that describe the problem
lambda = 15; %  Regularization parameter
N = 1000;    % Number of unknowns
M = 200;     % Number of measurements
K = 10;      % Signal sparsity

%% Construct sparse signal with K non-zeros
sig = zeros(N,1);
p = randperm(N);
sig(p(1:K))=1;

%% Construct random Gaussian matrix
D = randn(M,N);
D = D/norm(D);
%% Construct measurements
b = D*sig;
noise = randn(M,1)*max(abs(b))*.05;
b = b+noise;

%%  Call solver
[sol, out] = pdhg_scaledLasso( D,b,lambda);

%% Plot results
figure;
subplot(2,2,1);
plot(sig);
title('Original Signal');

subplot(2,2,2);
plot(b);
title('Measurements');

subplot(2,2,3);
plot(sol);
title('Recovered');

%  Plot the residuals, which measure convergence
subplot(2,2,4);
semilogy(out.p,'b');
hold on;
semilogy(out.d,'r');
hold off;
legend('Primal Residual','Dual Residual');
title('Convergence Curves');
